package prt2.age.graph;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author trust56
 */
public abstract class LocatedGraph<T> extends Graph<T, LocatedNode<T>> implements Serializable {

    protected final int NODE_RADIUS = 10;
    protected LocatedNode<T> selectedNode;
    protected Arc<T, LocatedNode<T>> selectedArc;

    public static boolean pointsCloserThan(double distance,
            Point point1, Point point2) {
        if (point1 == null || point2 == null) {
            return false;
        } else {
            int dx = Math.abs(point1.x - point2.x);
            int dy = Math.abs(point1.y - point2.y);
            return distance * distance > dx * dx + dy * dy;
        }
    }

    public void addArc(Point point) {
        if (selectedNode != null) {
            Arc<T, LocatedNode<T>> arc = new Arc<T, LocatedNode<T>>();
            arc.edge1 = selectedNode;
            for (LocatedNode<T> locatedNode : nodes) {
                if (pointsCloserThan(NODE_RADIUS, point, new Point(locatedNode.x, locatedNode.y))) {
                    arc.edge2 = locatedNode;
                }
            }
            if (arc.edge2 != null) {
                if (arc.edge1.equals(arc.edge2) && !allowLoopArcs) {
                    return;
                }
                arcs.add(arc);
                arc.edge1.arcs.add(arc);
                arc.edge2.arcs.add(arc);
                graphChanged();
            }
        }
    }

    public void addNode(int x, int y, T e) {
        LocatedNode<T> node = new LocatedNode<T>();
        node.x = x;
        node.y = y;
        node.element = e;
        for (LocatedNode<T> locatedNode : nodes) {
            if (pointsCloserThan(NODE_RADIUS,
                    new Point(locatedNode.x, locatedNode.y), new Point(x, y))) {
                return;
            }
        }
        nodes.add(node);
        graphChanged();
    }

    public void deleteSelectedNodeOrArc() {
        if (selectedNode != null) {
            List<Arc<T, LocatedNode<T>>> arcsToDelete = new ArrayList<Arc<T, LocatedNode<T>>>();
            for (Arc<T, LocatedNode<T>> arc : arcs) {
                if (arc.edge1 == selectedNode || arc.edge2 == selectedNode) {
                    arcsToDelete.add(arc);
                }
            }
            for (Arc<T, LocatedNode<T>> arc : arcsToDelete) {
                arcs.remove(arc);
            }
            nodes.remove(selectedNode);
            selectedNode = null;
            graphChanged();
        }
        if (selectedArc != null) {
            selectedArc.edge1.arcs.remove(selectedArc);
            selectedArc.edge2.arcs.remove(selectedArc);
            arcs.remove(selectedArc);
            graphChanged();
        }
    }

    public void dragPoint(Point point) {
        if (selectedNode != null) {
            selectedNode.x = point.x;
            selectedNode.y = point.y;
            graphChanged();
        }
    }

    public abstract void graphChanged();

    public void selectPoint(Point point) {
        LocatedNode<T> oldHighlightedNode = selectedNode;
        Arc<T, LocatedNode<T>> oldHighlightedArc = selectedArc;
        selectedNode = null;
        selectedArc = null;
        for (Arc<T, LocatedNode<T>> arc : arcs) {
            double tangent1 = (arc.edge1.x - arc.edge2.x)
                    / (double) (arc.edge1.y - arc.edge2.y);
            double tangent2 = (point.x - arc.edge2.x)
                    / (double) (point.y - arc.edge2.y);
            double tangent3 = (point.x - arc.edge1.x)
                    / (double) (point.y - arc.edge1.y);
            boolean ok = Math.abs(tangent1 - tangent2) < 0.2 && Math.abs(tangent1 - tangent3) < 0.2;
            //ok = ok && (arc.edge1.x > arc.edge2.x) ? arc.edge1.x > point.x : arc.edge1.x < point.x;
            if (ok) {
                selectedArc = arc;
                if (oldHighlightedArc != null && oldHighlightedArc.equals(selectedArc)) {
                    selectedArc = null;
                }
            }
        }
        for (LocatedNode<T> locatedNode : nodes) {
            if (pointsCloserThan(NODE_RADIUS,
                    point, new Point(locatedNode.x, locatedNode.y))) {
                selectedNode = locatedNode;
                if (oldHighlightedNode != null && selectedNode.equals(oldHighlightedNode)) {
                    selectedNode = null;
                }
                selectedArc = null;
            }
        }
        if (selectedNode != oldHighlightedNode
                || selectedArc != oldHighlightedArc) {
            graphChanged();
        }
    }

    public void setElementInSelectedNode(T element) {
        if (selectedNode != null) {
            selectedNode.element = element;
            graphChanged();
        }
    }

    protected int distance(Point point, LocatedNode<T> node) {
        int dx = point.x - node.x;
        int dy = point.y - node.y;
        return (int) Math.round(Math.sqrt(dx*dx + dy*dy));
    }
}
