package prt2.age.gui;

import android.app.Activity;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Toast;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import prt2.age.file.FileHandler;
import prt2.age.graph.LocatedGraph;
import prt2.age.graph.Point;

/**
 *
 * @author trust56
 */
public class GraphView extends View {

    private AndroidGraph<Character> graph;
    private Paint p;
    private LoaderThread loaderThread;
    private int mode;
    private static final int MODE_PUT_NODES = 1;
    private static final int MODE_PUT_ARCS = 2;
    private static final int MODE_MOVE_NODE = 3;

    public GraphView(Context context) {
        super(context);
        initGraphView();
    }

    public GraphView(Context context, AttributeSet attrs) {
        super(context, attrs);
        initGraphView();
    }

    public GraphView(Context context, AttributeSet attrs, int defStyle) {
        super(context, attrs, defStyle);
        initGraphView();
    }

    private void initGraphView() {
        setFocusable(true);
        graph = new AndroidGraph<Character>();
        graph.setView(this);
        graph.setAllowLoopArcs(false);
        mode = MODE_PUT_NODES;
        p = new Paint();
        p.setColor(Color.WHITE);
        FileHandler.getInstance().setContext(getContext());
        loaderThread = new LoaderThread();
        loaderThread.start();
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);
        canvas.drawRect(0, 0, getWidth(), getHeight(), p);
        if (graph != null && !loaderThread.isAlive()) {
            graph.draw(canvas);
        }
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        float tx = event.getAxisValue(MotionEvent.AXIS_X);
        float ty = event.getAxisValue(MotionEvent.AXIS_Y);
        int x = (int) tx;
        int y = (int) ty;
        Point point = new Point(x, y);
        switch (mode) {
            case MODE_PUT_NODES:
                graph.addNode(x, y, ' ');
                break;
            case MODE_PUT_ARCS:
                graph.addArc(point);
                break;
            case MODE_MOVE_NODE:
                graph.dragPoint(point);
                return true;
        }
        graph.selectPoint(point);
        return super.onTouchEvent(event);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode >= KeyEvent.KEYCODE_A && keyCode <= KeyEvent.KEYCODE_Z) {
            graph.setElementInSelectedNode((char) (keyCode - KeyEvent.KEYCODE_A + (int) 'a'));
        }
        return super.onKeyDown(keyCode, event);
    }

    void save() {
        try {
            graph.setView(null);
            FileHandler.getInstance().save(graph);
            graph.setView(this);
        } catch (FileNotFoundException ex) {
            showError("File cannot be opened", ex);
        } catch (IOException ex) {
            showError("Other IO error", ex);
        }
    }

    private void showError(final String message, final Throwable ex) {
        ((Activity) getContext()).runOnUiThread(new Runnable() {
            public void run() {
                Toast.makeText(getContext(), message + "\n" + ex.getMessage(), 5000).show();
            }
        });
    }

    boolean onOptionsItemSelected(MenuItem item) {
        if (item.isCheckable()) {
            item.setChecked(!item.isChecked());
        }
        switch (item.getItemId()) {
            case R.id.menu_save:
                save();
                return true;
            case R.id.menu_putnodes:
                if (item.isChecked()) {
                    mode = MODE_PUT_NODES;
                }
                return true;
            case R.id.menu_putarcs:
                if (item.isChecked()) {
                    mode = MODE_PUT_ARCS;
                }
                return true;
            case R.id.menu_move:
                if (item.isChecked()) {
                    mode = MODE_MOVE_NODE;
                }
                return true;
            case R.id.menu_del:
                graph.deleteSelectedNodeOrArc();
                return true;
            default:
                return false;
        }
    }

    private class LoaderThread extends Thread {

        @Override
        public void run() {
            try {
                LocatedGraph<Character> loadedGraph = FileHandler.getInstance().load();
                if (loadedGraph != null) {
                    graph = (AndroidGraph<Character>) loadedGraph;
                    graph.setView(GraphView.this);
                    postInvalidate();
                }
            } catch (StreamCorruptedException ex) {
                showError("The filestream is corrupted", ex);
            } catch (OptionalDataException ex) {
                showError("Optional data error", ex);
            } catch (ClassNotFoundException ex) {
                showError("Class not found", ex);
            } catch (IOException ex) {
                showError("Other IO error", ex);
            }
        }
    }
}
